local _, RS = ...
RS.Events = RS.Events or {}
local Events = RS.Events

-- local references - WoW and Lua
local CombatLogGetCurrentEventInfo = CombatLogGetCurrentEventInfo
local UnitName = UnitName
local playerName = UnitName("player")

-- local references - Addon
local Common = nil
local Variables = nil
local ActionType = nil
local bossList = nil
local AttemptReaction = nil
local GetSpellSettings = nil
local HasAura = nil

function Events:OnLoadCombat()
	Common = RS.Common
	ActionType = Common.ActionType
	bossList = Common.bossList
	HasAura = Common.HasAura
	Variables = RS.Variables
	AttemptReaction = RS.Reactions.AttemptReaction
	GetSpellSettings = RS.Reactions.GetSpellSettings
end

----------------------------------------------------------------------------------------------------
-- Data for handling events
----------------------------------------------------------------------------------------------------
-- counters of how many of each type of spell effect is on the player
Events.playerSpellEffects = {
	[1]  = 0, -- 12:stun + 14:knockout
	[2]  = 0, -- 5:fear + 24:horror + 23:turn
	[3]  = 0, -- 15:bleed
	[4]  = 0, -- 7:root
	[5]  = 0, -- 17:polymorph
	[6]  = 0, -- 18:banish
	[7]  = 0, -- 2:disoriented
	[8]  = 0, -- 11:snare + 27:daze
	[9]  = 0, -- 30:sapped
	[10] = 0, -- 1:charm
	[11] = 0, -- 9:silence
	[12] = 0, -- 10:sleep
	[13] = 0, -- 13:freeze
	[14] = 0, -- 29:immune_shield
	[15] = 0, -- 16:bandage
	[16] = 0, -- 19:shield
	[17] = 0, -- 3:disarm
}

-- totems are treated like people in TBC, so make a list of their NPC IDs to handle them as objects
-- TODO: not sure if they're treated like that in Classic
local totemMobList = {
	--[[earthbind totem        ]] [2630]=1,
	--[[fire nova totem        ]] [5879]=1, [6110]=1, [6111]=1, [7844]=1, [7845]=1,
	--[[magma totem            ]] [5929]=1, [7464]=1, [7465]=1, [7466]=1,
	--[[searing totem          ]] [2523]=1, [3902]=1, [3903]=1, [3904]=1, [7400]=1, [7402]=1,
	--[[fire resistance totem  ]] [5927]=1, [7424]=1, [7425]=1,
	--[[frost resistance totem ]] [5926]=1, [7412]=1, [7413]=1,
	--[[grace of air totem     ]] [7486]=1, [7487]=1, [15463]=1,
	--[[grounding totem        ]] [5925]=1,
	--[[nature resistance totem]] [7467]=1, [7468]=1, [7469]=1,
	--[[stoneskin totem        ]] [5873]=1, [5919]=1, [5920]=1, [7366]=1, [7367]=1, [7368]=1,
	--[[windwall totem         ]] [9687]=1, [9688]=1, [9689]=1,
	--[[strength of earth totem]] [5874]=1, [5921]=1, [5922]=1, [7403]=1, [15464]=1,
	--[[windfury totem         ]] [6112]=1, [7483]=1, [7484]=1,
	--[[mana spring totem      ]] [3573]=1, [7414]=1, [7415]=1, [7416]=1,
	--[[healing stream totem   ]] [3527]=1, [3906]=1, [3907]=1, [3908]=1, [3909]=1,
	--[[mana tide totem        ]] [10467]=1, [11100]=1, [11101]=1,
	--[[disease cleansing totem]] [5924]=1,
	--[[poison cleansing totem ]] [5923]=1,
	--[[tranquil air totem     ]] [15803]=1,
	--[[tremor totem           ]] [5913]=1,
}

--------------------------------------------------
-- tables used to check a spell's effect
--------------------------------------------------
local spellEffectList = {
	-- charm
	[605]=1, [1098]=1, [6358]=1, [7645]=1, [8345]=1, [10911]=1, [10912]=1, [11446]=1, [11725]=1, [11726]=1, [12888]=1, [13181]=1, [14515]=1, [15859]=1, [16053]=1, [17405]=1, [19469]=1, [20604]=1, [20740]=1, [20882]=1, [24327]=1, [26740]=1, [29490]=1, [29516]=1, [29546]=1, [30850]=1, [31865]=1, [32764]=1, [33502]=1, [35120]=1, [35280]=1, [36241]=1, [36274]=1, [36661]=1, [36866]=1, [37122]=1, [37162]=1, [37200]=1, [38626]=1, [38915]=1, [41345]=1, [43550]=1, [44547]=1, [46427]=1,
	-- disoriented
	[2094]=2, [26108]=2, [42805]=2,
	-- disarm
	[676]=3, [5259]=3, [6608]=3, [6713]=3, [8379]=3, [10851]=3, [11879]=3, [13534]=3, [14180]=3, [15752]=3, [22419]=3, [22691]=3, [23365]=3, [25057]=3, [25655]=3, [27581]=3, [30013]=3, [31955]=3, [33126]=3, [35055]=3, [36207]=3, [36208]=3, [36209]=3, [36510]=3, [39489]=3, [41054]=3, [45205]=3, [45206]=3,
	-- fear
	[1513]=5, [3109]=5, [5134]=5, [5246]=5, [5484]=5, [5782]=5, [6213]=5, [6215]=5, [6605]=5, [7093]=5, [7399]=5, [8122]=5, [8124]=5, [8225]=5, [8715]=5, [10888]=5, [10890]=5, [12096]=5, [12542]=5, [13704]=5, [14100]=5, [14326]=5, [14327]=5, [16096]=5, [16508]=5, [17928]=5, [18431]=5, [19134]=5, [19408]=5, [20511]=5, [21330]=5, [21869]=5, [21898]=5, [22678]=5, [22686]=5, [22884]=5, [23275]=5, [25260]=5, [25815]=5, [26042]=5, [26070]=5, [26580]=5, [26641]=5, [26661]=5, [27610]=5, [27641]=5, [27990]=5, [28315]=5, [29168]=5, [29321]=5, [29419]=5, [29544]=5, [29685]=5, [30002]=5, [30530]=5, [30584]=5, [30615]=5, [30752]=5, [31013]=5, [31358]=5, [31365]=5, [31404]=5, [31970]=5, [32040]=5, [32241]=5, [32421]=5, [33547]=5, [33789]=5, [33829]=5, [33924]=5, [34259]=5, [34322]=5, [35198]=5, [35474]=5, [36629]=5, [36922]=5, [36950]=5, [37939]=5, [38154]=5, [38258]=5, [38595]=5, [38660]=5, [38945]=5, [38946]=5, [39048]=5, [39119]=5, [39176]=5, [39210]=5, [39415]=5, [39427]=5, [40221]=5, [40454]=5, [40636]=5, [41150]=5, [41436]=5, [42690]=5, [43432]=5, [43590]=5, [44863]=5, [46561]=5,
	-- root
	[113]=7, [339]=7, [512]=7, [745]=7, [1062]=7, [3542]=7, [4962]=7, [5195]=7, [5196]=7, [5567]=7, [6533]=7, [8142]=7, [8312]=7, [8346]=7, [8377]=7, [9852]=7, [9853]=7, [9915]=7, [10017]=7, [10852]=7, [11264]=7, [11820]=7, [11831]=7, [11922]=7, [12023]=7, [12024]=7, [12252]=7, [12494]=7, [12674]=7, [12747]=7, [12748]=7, [13099]=7, [13119]=7, [13138]=7, [13608]=7, [14030]=7, [14907]=7, [15063]=7, [15474]=7, [15531]=7, [15532]=7, [15609]=7, [16469]=7, [16566]=7, [19185]=7, [19229]=7, [19970]=7, [19971]=7, [19972]=7, [19973]=7, [19974]=7, [19975]=7, [20654]=7, [20699]=7, [21331]=7, [22127]=7, [22415]=7, [22519]=7, [22645]=7, [22744]=7, [22800]=7, [22924]=7, [22994]=7, [23694]=7, [24110]=7, [24648]=7, [25999]=7, [26071]=7, [26989]=7, [27010]=7, [28297]=7, [28858]=7, [28991]=7, [29849]=7, [29991]=7, [30094]=7, [31287]=7, [31290]=7, [31409]=7, [31983]=7, [32173]=7, [32192]=7, [32365]=7, [32859]=7, [33844]=7, [34326]=7, [34725]=7, [34740]=7, [34746]=7, [34779]=7, [34782]=7, [35107]=7, [35234]=7, [35247]=7, [35831]=7, [35963]=7, [36786]=7, [36827]=7, [36989]=7, [37480]=7, [37823]=7, [38033]=7, [38051]=7, [38316]=7, [38338]=7, [38661]=7, [38843]=7, [38900]=7, [38912]=7, [39035]=7, [39063]=7, [39584]=7, [40082]=7, [40333]=7, [40363]=7, [40727]=7, [41580]=7, [43362]=7, [43426]=7, [43585]=7, [44177]=7, [45825]=7, [45905]=7, [46555]=7, [47168]=7, [50762]=7,
	-- silence
	[1330]=9, [3589]=9, [6726]=9, [6942]=9, [7074]=9, [8281]=9, [8988]=9, [9552]=9, [12528]=9, [12946]=9, [15487]=9, [16838]=9, [18278]=9, [18327]=9, [18425]=9, [18469]=9, [18498]=9, [19393]=9, [19821]=9, [22666]=9, [23207]=9, [23918]=9, [24259]=9, [24687]=9, [25046]=9, [26069]=9, [27559]=9, [28730]=9, [29505]=9, [29904]=9, [29943]=9, [30225]=9, [30849]=9, [31015]=9, [31069]=9, [31273]=9, [31344]=9, [33390]=9, [33686]=9, [33913]=9, [34087]=9, [34088]=9, [34089]=9, [34922]=9, [35892]=9, [36022]=9, [36297]=9, [37031]=9, [37160]=9, [38491]=9, [38913]=9, [39052]=9, [40823]=9, [42201]=9, [42205]=9,
	-- sleep
	[700]=10, [1090]=10, [2637]=10, [3636]=10, [7967]=10, [8040]=10, [8399]=10, [8901]=10, [8902]=10, [9159]=10, [12098]=10, [15822]=10, [15970]=10, [16798]=10, [18657]=10, [18658]=10, [19386]=10, [20663]=10, [20669]=10, [20989]=10, [24004]=10, [24132]=10, [24133]=10, [24335]=10, [24360]=10, [27068]=10, [28504]=10, [29148]=10, [29679]=10, [31292]=10, [31541]=10, [34039]=10, [34801]=10, [36333]=10, [36402]=10, [37990]=10, [38510]=10, [38886]=10, [41186]=10, [41396]=10,
	-- snare
	[89]=11, [246]=11, [3409]=11, [3600]=11, [3604]=11, [5116]=11, [5159]=11, [6136]=11, [6146]=11, [6984]=11, [7279]=11, [7321]=11, [7992]=11, [8078]=11, [8147]=11, [8716]=11, [8732]=11, [9080]=11, [9462]=11, [10855]=11, [10987]=11, [11201]=11, [11436]=11, [12323]=11, [12484]=11, [12485]=11, [12486]=11, [12531]=11, [12551]=11, [12705]=11, [13496]=11, [13747]=11, [13810]=11, [14897]=11, [15548]=11, [15588]=11, [16050]=11, [16568]=11, [17134]=11, [17165]=11, [17174]=11, [18099]=11, [18101]=11, [18118]=11, [18223]=11, [18328]=11, [18802]=11, [18972]=11, [19137]=11, [19496]=11, [22356]=11, [22639]=11, [22914]=11, [22919]=11, [23600]=11, [23931]=11, [23953]=11, [24225]=11, [24415]=11, [25022]=11, [25603]=11, [25809]=11, [26078]=11, [26141]=11, [26143]=11, [26211]=11, [26379]=11, [26554]=11, [27634]=11, [27640]=11, [27993]=11, [29292]=11, [29407]=11, [29539]=11, [29540]=11, [29570]=11, [29667]=11, [29703]=11, [29990]=11, [30035]=11, [30109]=11, [30494]=11, [30633]=11, [30981]=11, [30984]=11, [30989]=11, [31125]=11, [31257]=11, [31467]=11, [31473]=11, [31478]=11, [31553]=11, [31589]=11, [31741]=11, [32000]=11, [32013]=11, [32024]=11, [32065]=11, [32317]=11, [32417]=11, [32651]=11, [32921]=11, [32922]=11, [33061]=11, [33628]=11, [33967]=11, [35032]=11, [35101]=11, [35240]=11, [35244]=11, [35263]=11, [35351]=11, [35493]=11, [35507]=11, [35542]=11, [35545]=11, [35546]=11, [35547]=11, [35548]=11, [35919]=11, [36148]=11, [36214]=11, [36414]=11, [36415]=11, [36416]=11, [36417]=11, [36448]=11, [36457]=11, [36458]=11, [36464]=11, [36474]=11, [36475]=11, [36508]=11, [36518]=11, [36580]=11, [36659]=11, [36706]=11, [36824]=11, [36839]=11, [36843]=11, [36974]=11, [37276]=11, [37330]=11, [37359]=11, [37478]=11, [37591]=11, [37621]=11, [37654]=11, [37986]=11, [38243]=11, [38256]=11, [38262]=11, [38537]=11, [38663]=11, [38767]=11, [38821]=11, [38822]=11, [38880]=11, [38985]=11, [38986]=11, [38987]=11, [38988]=11, [38989]=11, [38990]=11, [38995]=11, [39049]=11, [39538]=11, [39665]=11, [39900]=11, [41086]=11, [41264]=11, [41978]=11, [42396]=11, [43130]=11, [43131]=11, [43357]=11, [43530]=11, [43583]=11, [44033]=11, [44120]=11, [44289]=11, [45195]=11, [46434]=11, [46745]=11, [47106]=11,
	-- stun
	[45]=12, [56]=12, [408]=12, [835]=12, [853]=12, [1833]=12, [2880]=12, [3143]=12, [3242]=12, [3263]=12, [3446]=12, [3551]=12, [3609]=12, [3635]=12, [5106]=12, [5164]=12, [5211]=12, [5276]=12, [5403]=12, [5530]=12, [5588]=12, [5589]=12, [5648]=12, [5649]=12, [5703]=12, [5708]=12, [5918]=12, [6253]=12, [6266]=12, [6304]=12, [6409]=12, [6435]=12, [6466]=12, [6524]=12, [6607]=12, [6728]=12, [6730]=12, [6749]=12, [6798]=12, [6927]=12, [6945]=12, [6982]=12, [7139]=12, [7803]=12, [7922]=12, [7964]=12, [8150]=12, [8151]=12, [8208]=12, [8242]=12, [8285]=12, [8391]=12, [8643]=12, [8646]=12, [8983]=12, [10308]=12, [10856]=12, [11020]=12, [11428]=12, [11430]=12, [11650]=12, [11836]=12, [11876]=12, [12355]=12, [12461]=12, [12734]=12, [12798]=12, [12809]=12, [13005]=12, [13237]=12, [13902]=12, [14102]=12, [14902]=12, [15239]=12, [15269]=12, [15283]=12, [15398]=12, [15535]=12, [15593]=12, [15618]=12, [15621]=12, [15652]=12, [15655]=12, [15743]=12, [15753]=12, [15847]=12, [15878]=12, [16075]=12, [16104]=12, [16350]=12, [16497]=12, [16600]=12, [16727]=12, [16740]=12, [16790]=12, [16803]=12, [16869]=12, [16922]=12, [17011]=12, [17276]=12, [17286]=12, [17293]=12, [17308]=12, [17500]=12, [18093]=12, [18103]=12, [18144]=12, [18395]=12, [18763]=12, [18812]=12, [19128]=12, [19136]=12, [19364]=12, [19410]=12, [19482]=12, [19641]=12, [19780]=12, [19798]=12, [20170]=12, [20276]=12, [20277]=12, [20310]=12, [20549]=12, [20683]=12, [20685]=12, [21099]=12, [21152]=12, [21748]=12, [21749]=12, [21808]=12, [21990]=12, [22289]=12, [22427]=12, [22592]=12, [22692]=12, [22915]=12, [23103]=12, [23364]=12, [23454]=12, [23919]=12, [24213]=12, [24333]=12, [24375]=12, [24394]=12, [24600]=12, [24671]=12, [25056]=12, [25189]=12, [25654]=12, [25852]=12, [27615]=12, [27758]=12, [27880]=12, [28125]=12, [28314]=12, [28445]=12, [28725]=12, [29670]=12, [29676]=12, [29690]=12, [29711]=12, [29896]=12, [30621]=12, [30688]=12, [30732]=12, [30761]=12, [30832]=12, [30986]=12, [31046]=12, [31274]=12, [31286]=12, [31367]=12, [31368]=12, [31390]=12, [31408]=12, [31422]=12, [31480]=12, [31610]=12, [31718]=12, [31719]=12, [31755]=12, [31819]=12, [31843]=12, [31864]=12, [31964]=12, [31994]=12, [32015]=12, [32023]=12, [32109]=12, [32361]=12, [32416]=12, [32588]=12, [32654]=12, [32752]=12, [32864]=12, [32905]=12, [33487]=12, [33709]=12, [33781]=12, [33792]=12, [33919]=12, [34108]=12, [34243]=12, [34267]=12, [34357]=12, [34510]=12, [34716]=12, [34752]=12, [34885]=12, [35011]=12, [35202]=12, [35238]=12, [35313]=12, [35492]=12, [35783]=12, [35856]=12, [36138]=12, [36254]=12, [36809]=12, [36835]=12, [36877]=12, [36924]=12, [36929]=12, [37012]=12, [37029]=12, [37369]=12, [37592]=12, [37991]=12, [38169]=12, [38357]=12, [38682]=12, [38737]=12, [38911]=12, [39002]=12, [39017]=12, [39021]=12, [39077]=12, [39157]=12, [39229]=12, [39313]=12, [39796]=12, [39865]=12, [40077]=12, [40184]=12, [40262]=12, [40380]=12, [40846]=12, [40864]=12, [40936]=12, [41182]=12, [41274]=12, [41356]=12, [41358]=12, [41389]=12, [41468]=12, [41534]=12, [43437]=12, [44415]=12, [44799]=12, [45065]=12, [45122]=12, [46024]=12, [46026]=12, [46183]=12, [46184]=12, [46288]=12,
	-- freeze
	[3355]=13, [14308]=13, [14309]=13, [34973]=13, [36527]=13, [36911]=13, [41590]=13, [43448]=13,
	-- knockout
	[12540]=14, [13327]=14, [13579]=14, [15091]=14, [15744]=14, [16046]=14, [17145]=14, [17277]=14, [20066]=14, [22424]=14, [23039]=14, [23113]=14, [24698]=14, [30600]=14, [30980]=14, [32779]=14, [34940]=14, [36862]=14, [38064]=14, [38863]=14,
	-- bleed
	[703]=15, [772]=15, [1079]=15, [1943]=15, [3147]=15, [4102]=15, [4244]=15, [5597]=15, [5598]=15, [6546]=15, [6547]=15, [6548]=15, [8631]=15, [8632]=15, [8633]=15, [8639]=15, [8640]=15, [8818]=15, [9007]=15, [9492]=15, [9493]=15, [9752]=15, [9824]=15, [9826]=15, [9894]=15, [9896]=15, [10266]=15, [11273]=15, [11274]=15, [11275]=15, [11289]=15, [11290]=15, [11572]=15, [11573]=15, [11574]=15, [11977]=15, [12054]=15, [12162]=15, [12721]=15, [12850]=15, [12868]=15, [13318]=15, [13443]=15, [13445]=15, [13738]=15, [14087]=15, [14118]=15, [14331]=15, [14874]=15, [14903]=15, [15583]=15, [15976]=15, [16095]=15, [16393]=15, [16403]=15, [16406]=15, [16509]=15, [17153]=15, [17504]=15, [18075]=15, [18078]=15, [18106]=15, [18200]=15, [18202]=15, [19771]=15, [21949]=15, [23256]=15, [24192]=15, [24331]=15, [24332]=15, [25208]=15, [26839]=15, [26867]=15, [26884]=15, [27007]=15, [27008]=15, [27555]=15, [27556]=15, [27638]=15, [28913]=15, [29574]=15, [29578]=15, [29583]=15, [29906]=15, [29915]=15, [29935]=15, [30069]=15, [30070]=15, [30285]=15, [30639]=15, [31041]=15, [31410]=15, [31956]=15, [32019]=15, [32901]=15, [33865]=15, [33912]=15, [35144]=15, [35321]=15, [36023]=15, [36054]=15, [36332]=15, [36383]=15, [36450]=15, [36590]=15, [36617]=15, [36789]=15, [36965]=15, [36991]=15, [37066]=15, [37487]=15, [37662]=15, [37937]=15, [37973]=15, [38056]=15, [38363]=15, [38772]=15, [38801]=15, [38810]=15, [38848]=15, [39198]=15, [39215]=15, [39382]=15, [40199]=15, [41092]=15, [41932]=15, [43093]=15, [43100]=15, [43104]=15, [43246]=15,
	-- bandage
	[746]=16, [1159]=16, [3267]=16, [3268]=16, [7926]=16, [7927]=16, [10838]=16, [10839]=16, [18608]=16, [18610]=16, [23567]=16, [23568]=16, [23569]=16, [23696]=16, [24412]=16, [24413]=16, [24414]=16, [27030]=16, [27031]=16, [30020]=16, [35207]=16, [36348]=16, [38919]=16, [40330]=16,
	-- polymorph
	[118]=17, [228]=17, [851]=17, [4060]=17, [10253]=17, [11641]=17, [12824]=17, [12825]=17, [12826]=17, [13323]=17, [14621]=17, [15534]=17, [16097]=17, [16707]=17, [16708]=17, [16709]=17, [17172]=17, [17738]=17, [18503]=17, [22274]=17, [22566]=17, [23603]=17, [24053]=17, [24708]=17, [24709]=17, [24710]=17, [24711]=17, [24712]=17, [24713]=17, [24717]=17, [24718]=17, [24719]=17, [24720]=17, [24723]=17, [24724]=17, [24732]=17, [24733]=17, [24735]=17, [24736]=17, [24737]=17, [24740]=17, [24741]=17, [26157]=17, [26272]=17, [26273]=17, [26274]=17, [27760]=17, [28271]=17, [28272]=17, [29044]=17, [29314]=17, [29848]=17, [30077]=17, [30501]=17, [30504]=17, [30506]=17, [30838]=17, [32826]=17, [33173]=17, [34639]=17, [36700]=17, [36840]=17, [38245]=17, [38896]=17, [40400]=17, [41334]=17, [43309]=17, [46280]=17, [46295]=17,
	-- banish
	[710]=18, [8994]=18, [16045]=18, [16451]=18, [18647]=18, [27565]=18, [30231]=18, [30940]=18, [31797]=18, [33786]=18, [35182]=18, [37546]=18, [38009]=18, [38456]=18, [38505]=18, [38791]=18, [39622]=18, [40578]=18, [43528]=18, [44765]=18,
	-- shield
	-- removed: [6788]=19 (weakened soul)
	[17]=19, [592]=19, [600]=19, [3747]=19, [6065]=19, [6066]=19, [10898]=19, [10899]=19, [10900]=19, [10901]=19, [20706]=19, [22752]=19, [25217]=19, [25218]=19, [27607]=19, [32504]=19, [34788]=19, [41363]=19, [44175]=19, [44291]=19, [46193]=19,
	-- turn
	[2878]=23, [5627]=23, [10326]=23, [32724]=23, [32725]=23, [35774]=23, [35775]=23, [45350]=23, [45352]=23,
	-- horror
	[33130]=24, [34984]=24,
	-- daze
	[32674]=27, [32774]=27, [35853]=27, [37554]=27, [38631]=27, [40019]=27, [46873]=27,
	-- immune shield
	[498]=29, [642]=29, [1020]=29, [1022]=29, [5573]=29, [5599]=29, [10278]=29, [27619]=29, [41367]=29, [45438]=29, [46604]=29,
	-- sapped
	[2070]=30, [6770]=30, [11297]=30,
}

-- a table to look up which events to use and which indice to use in self.playerSpellEffects
local spellEffectLookup = {
	[1]  = {10, "CharmBegin",     "CharmEnd"},
	[2]  = {7,  "DisorientBegin", "DisorientEnd"},
	[3]  = {17, "DisarmBegin",    "DisarmEnd"},
	[5]  = {5,  "FearBegin",      "FearEnd"},
	[7]  = {4,  "RootBegin",      "RootEnd"},
	[9]  = {11, "SilenceBegin",   "SilenceEnd"},
	[10] = {12, "SleepBegin",     "SleepEnd"},
	[11] = {8,  "SnareBegin",     "SnareEnd"},
	[12] = {1,  "StunBegin",      "StunEnd"},
	[13] = {13, "FreezeBegin",    "FreezeEnd"},
	[15] = {3,  "BleedBegin",     "BleedEnd"},
	[16] = {15, "BandageBegin",   "BandageEnd"},
	[17] = {5,  "PolymorphBegin", "PolymorphEnd"},
	[18] = {6,  "BanishBegin",    "BanishEnd"},
	[19] = {16, "ShieldBegin",    "ShieldEnd"},
	[29] = {14, "ImmunityBegin",  "ImmunityEnd"},
	[30] = {9,  "SappedBegin",    "SappedEnd"},
}
spellEffectLookup[14] = spellEffectLookup[12] -- knockout to stun
spellEffectLookup[23] = spellEffectLookup[5]  -- turn to fear
spellEffectLookup[24] = spellEffectLookup[5]  -- horror to fear
spellEffectLookup[27] = spellEffectLookup[11] -- daze to snare

-- return true if the player has any kind of incapacitating effect (stun/knockout/fear/etc) on them
function Events:IsPlayerIncapacitated()
	local e = self.playerSpellEffects
	return (e[1] > 0 or e[2] > 0 or e[4] > 0 or e[5] > 0 or e[6] > 0 or e[7] > 0 or e[9] > 0 or e[10] > 0 or e[12] > 0 or e[13] > 0)
end

----------------------------------------------------------------------------------------------------
-- Handling events
----------------------------------------------------------------------------------------------------
function Events:COMBAT_LOG_EVENT_UNFILTERED()
	local _, action, _, _, sourceName, sourceFlags, _, destGuid, destName, destFlags, _,
		ex1, ex2, ex3, ex4, ex5, ex6, ex7, ex8, ex9, ex10, ex11, ex12, ex13, ex14 = CombatLogGetCurrentEventInfo()
	if ex1 and ex1 == 0 then ex1 = nil end -- spell ID

	self.sourceName  = sourceName
	self.sourceFlags = sourceFlags
	self.destName  = destName
	self.destFlags = destFlags

	-- normal attack
	if action == "SWING_DAMAGE" or action == "RANGE_DAMAGE" then
		if ex7 then -- critical
			if destName == playerName then
				AttemptReaction("CriticallyHitByNormal", 0, nil, destName, sourceName, ActionType.HIT)
			else
				AttemptReaction("CriticalHitNormal", 0, nil, sourceName, destName, ActionType.HIT)
			end
		end
		return
	end

	-- handle spells that do damage/healing when hitting - both instant and cast-time spells
	if action == "SPELL_DAMAGE" or action == "SPELL_HEAL" then
		-- ex4 = the damage/heal amount
		if ex4 > 0 then
			local hitType
			if action == "SPELL_DAMAGE" then
				if     ex10 then hitType = "Critical"
				elseif ex11 then hitType = "Glancing"
				elseif ex12 then hitType = "Crushing"
				else             hitType = "Normal"
				end
			else
				hitType = ex7 and "Critical" or "Normal"
			end

			self:RemoveCastCheck(sourceName, ex2)
			if not AttemptReaction(ex2, ex1, hitType, sourceName, destName, ActionType.HIT) and hitType == "Critical" then
				-- try generic crit events - must set the current spell manually
				local Variables = Variables
				Variables.spellName = ex2
				Variables.spellId = ex1
				if destName == playerName then
					AttemptReaction(action == "SPELL_HEAL" and "CriticallyHitByHeal" or "CriticallyHitBySpell", 0, hitType, destName, sourceName, ActionType.HIT)
				else
					AttemptReaction(action == "SPELL_HEAL" and "CriticalHitHeal" or "CriticalHitSpell", 0, hitType, sourceName, destName, ActionType.HIT)
				end
			end
		end
		return
	end

	-- handle periodic damage/healing
	if action == "SPELL_PERIODIC_DAMAGE" then
		local hitType
		if     ex10 then hitType = "Critical"
		elseif ex11 then hitType = "Glancing"
		elseif ex12 then hitType = "Crushing"
		else             hitType = "Normal"
		end
		AttemptReaction(ex2, ex1, hitType, sourceName, destName, ActionType.PERIODIC)
		return
	end
	if action == "SPELL_PERIODIC_HEAL" then
		hitType = ex7 and "Critical" or "Normal"
		AttemptReaction(ex2, ex1, hitType, sourceName, destName, ActionType.PERIODIC)
		return
	end

	-- handle spells that are cast instantly and do no immediate damage/healing
	if action == "SPELL_CAST_SUCCESS" then
		local spellSettings = GetSpellSettings(ex2, ex1)
		if spellSettings then
			self:AddCastCheck(sourceName, destName, ex2, ex1)
		end
		return
	end

	-- for spells with a cast time that do no damage - save the spell being cast and the target for UNIT_SPELLCAST_SUCCEEDED to use
	if action == "SPELL_CAST_START" then
		local spellSettings = GetSpellSettings(ex2, ex1)
		if spellSettings then
			if sourceName == playerName then
				if not self.usedStartCast then
					self.usedStartCast = true
					self.playerCastingSpellId = ex1
					self.playerCastingSpellName = ex2
					-- playerCastingTarget set up during UNIT_SPELLCAST_SENT event
					AttemptReaction(ex2, ex1, nil, sourceName, self.playerCastingTarget, ActionType.START_CAST)
				end
			else
				AttemptReaction(ex2, ex1, nil, sourceName, nil, ActionType.START_CAST)
			end
		end
		return
	end

	-- handle procs - ignore them if the spell is currently being checked for success
	if action == "SPELL_ENERGIZE" or action == "SPELL_DRAIN" or action == "SPELL_LEECH" then
		if not self:HasCastCheck(sourceName, ex2) then
			AttemptReaction(ex2, ex1, nil, sourceName, destName, ActionType.HIT)
		end
		return
	end

	-- aura was added/refreshed on someone
	if action == "SPELL_AURA_APPLIED" or action == "SPELL_AURA_REFRESH" or action == "SPELL_AURA_APPLIED_DOSE" then
		AttemptReaction(ex2, ex1, nil, nil, destName, ActionType.AURA_GAINED)

		-- check generic spell effects like stun/fear/bleed
		if action == "SPELL_AURA_APPLIED" then
			local effect = spellEffectList[ex1]
			if effect then
				local effectInfo = spellEffectLookup[effect]
				if destName == playerName then
					self.playerSpellEffects[effectInfo[1]] = self.playerSpellEffects[effectInfo[1]] + 1
					if self.playerSpellEffects[effectInfo[1]] == 1 then
						AttemptReaction(effectInfo[2], ex1, nil, sourceName, destName, ActionType.EVENT_AURA)
					end
				else
					AttemptReaction(effectInfo[2], ex1, nil, nil, destName, ActionType.EVENT_AURA)
				end
			end
		end
		return
	end

	-- aura was removed from someone
	if action == "SPELL_AURA_REMOVED" then
		-- if they have any limitAura auras tracked, remove them
		local auras = self.trackedAuras[destName]
		if auras and not HasAura(destName, ex2) then
			auras[ex2] = nil
		end

		AttemptReaction(ex2, ex1, nil, nil, destName, ActionType.AURA_REMOVED)

		-- check generic spell effects like stun/fear/bleed
		local effect = spellEffectList[ex1]
		if effect then
			local effectInfo = spellEffectLookup[effect]
			if destName == playerName then
				local effectId = effectInfo[1]
				self.playerSpellEffects[effectId] = self.playerSpellEffects[effectId] - 1

				if self.playerSpellEffects[effectId] == 0 then
					AttemptReaction(effectInfo[3], ex1, nil, sourceName, destName, ActionType.EVENT_AURA)
				elseif self.playerSpellEffects[effectId] < 0 then
					self.playerSpellEffects[effectId] = 0
				end
			else
				AttemptReaction(effectInfo[3], ex1, nil, sourceName, destName, ActionType.EVENT_AURA)
			end
		end
		return
	end

	-- for non-damage spells there's a tiny delay before the end of the spell and considering it
	-- successful. If these happen during that tiny delay then it was a miss or resist.
	if action == "SPELL_MISSED" then
		-- ex4 = miss type
		self:RemoveCastCheck(sourceName, ex2)
		AttemptReaction(ex2, ex1, ex4, sourceName, destName, ActionType.MISS)
		return
	end

	-- interrupting someone's spell
	if action == "SPELL_INTERRUPT" then
		local Variables = Variables
		Variables.spellId        = ex1
		Variables.spellName      = ex2
		Variables.extraSpellId   = ex4
		Variables.extraSpellName = ex5
		if destName == playerName then
			AttemptReaction("Interrupted", 0, nil, destName, sourceName, ActionType.HIT)
		else
			AttemptReaction("Interrupting", 0, nil, sourceName, destName, ActionType.HIT)
		end
		return
	end

	-- handle some spells that create or summon things
	if action == "SPELL_CREATE" or action == "SPELL_SUMMON" then
		AttemptReaction(ex2, ex1, nil, sourceName, destName, ActionType.HIT)
		return
	end

	-- killing blow by you or someone in the group - doesn't work for other raid subgroups
	if action == "PARTY_KILL" then
		-- find someone looking at the mob (or same-named one) to figure out if it's a boss or not
		local classification = bossList[destName] and "worldboss"
		if not classification then
			if UnitName("target") == destName then
				classification = UnitClassification("target")
			else
				if IsInRaid() then
					for i=1,40 do
						if UnitName("raid" .. i .. "target") == destName then
							classification = UnitClassification("party" .. i .. "target")
							break
						end
					end
				else
					for i=1,4 do
						if UnitName("party" .. i .. "target") == destName then
							classification = UnitClassification("party" .. i .. "target")
							break
						end
					end
				end
			end
		end
		if classification then
			if classification == "worldboss" then
				AttemptReaction("KillingBlowBoss", 0, nil, sourceName, destName, ActionType.HIT)
			else
				AttemptReaction("KillingBlow", 0, nil, sourceName, destName, ActionType.HIT)
			end
		end
		return
	end

	-- something died
	if action == "UNIT_DIED" then
		-- specific named deaths first
		if AttemptReaction(destName, 0, nil, playerName, destName, ActionType.HIT) then
			return
		end

		-- generic death events if no specific name was found to react to
		-- get the NPC ID of the creature to check if it's a totem or similar object
		local targetId = tonumber(destGuid:sub(9, 12), 16)
		if targetId ~= 0 and totemMobList[targetId] then
			AttemptReaction("DeathTotem", 0, nil, destName, nil, ActionType.HIT)
		else
			if destName == playerName then
				self.onLowHealth = 0
				self.onLowMana   = 0
			end
			AttemptReaction("Death", 0, nil, destName, nil, ActionType.HIT)
		end

		self.trackedAuras[destName] = nil
		return
	end

	-- someone took environmental damage - slime/fatigue not tracked because they're broken/rare
	if action == "ENVIRONMENTAL_DAMAGE" then
		-- ex1 = damageType
		-- ex2 = damageAmount
		if ex1 then
			local eventType = nil
			local health = UnitHealth(destName)
			local death = (health and ex2 >= health)
			if ex1 == "Falling" then
				if death then
					eventType = "EnvironmentFallDeath"
				else
					-- only show if losing a certain health pencentage (or amount if that can't be seen)
					local max = UnitHealthMax(destName)
					if max then
						if ex2 * 100 / max < 30 then
							return
						end
					elseif ex2 < 1800 then
						return
					end
					eventType = "EnvironmentFall"
				end
			elseif ex1 == "Fire" then
				eventType = death and "EnvironmentFireDeath" or "EnvironmentFire"
			elseif ex1 == "Lava" then
				eventType = death and "EnvironmentLavaDeath" or "EnvironmentLava"
			elseif ex1 == "Slime" then
				eventType = death and "EnvironmentSlimeDeath" or "EnvironmentSlime"
			elseif ex1 == "Drowning" then
				eventType = death and "EnvironmentDrownDeath" or "EnvironmentDrown"
			end

			if eventType then
				AttemptReaction(eventType, 0, nil, destName, nil, ActionType.HIT)
			end
		end
		return
	end
end

--------------------------------------------------
-- more spell casting handling
--------------------------------------------------
-- handle spells with a cast time that do no damage
function Events:UNIT_SPELLCAST_SUCCEEDED(unitId, castGuid, spellId)
	self.usedStartCast = false
	if unitId == "player" then
		local spellName = (GetSpellInfo(spellId))
		if self.playerCastingSpellId == spellId or self.playerCastingSpellName == spellName then
			self:AddCastCheck(playerName, self.playerCastingTarget, spellName, self.playerCastingSpellId)
		elseif spellName == self.DRINK then
			AttemptReaction("ConsumeDrink", 0, nil, playerName, nil, ActionType.HIT)
		elseif spellName == self.FOOD then
			AttemptReaction("ConsumeFood", 0, nil, playerName, nil, ActionType.HIT)
		elseif spellName == self.STRONG_ALCOHOL or spellName == self.WEAK_ALCOHOL or spellName == self.STANDARD_ALCOHOL
			or spellName == self.POTENT_ALCOHOL or spellName == self.MOONGLOW_ALCOHOL or spellName == self.DARK_IRON_ALCOHOL then
			AttemptReaction("ConsumeAlcohol", 0, nil, playerName, nil, ActionType.HIT)
		end
	end
end

-- a cast has ended
function Events:UNIT_SPELLCAST_FAILED(unitId)
	if arg1 == "player" then
		self.usedStartCast = false
	end
end

function Events:UNIT_SPELLCAST_INTERRUPTED(unitId)
	if arg1 == "player" then
		self.usedStartCast = false
	end
end

-- opening an object, or save the player's target because it's not available in SPELLCAST_START events
function Events:UNIT_SPELLCAST_SENT(unitId, target, castGuid, spellId)
	-- if target == self.OPENING then
	-- 	AttemptReaction("OpeningWorldObject", 0, nil, playerName, target, ActionType.HIT)
	-- else
	self.playerCastingTarget = target
	-- end
end

-- the player started casting a spell - most spells will use the combat event SPELL_CAST_START
-- action first, but some won't so this has to be used. The spell ID isn't known in these cases.
function Events:UNIT_SPELLCAST_START(unitId, castGuid, spellId)
	if not self.usedStartCast and unitId == "player" then
		self.usedStartCast = true
		local spellName = (GetSpellInfo(spellId))
		local spellSettings = GetSpellSettings(spellName, spellId)
		if spellSettings then
			self.playerCastingSpellName = spellName
			self.playerCastingSpellId = spellId
			-- playerCastingTarget set up during UNIT_SPELLCAST_SENT event
			AttemptReaction(self.playerCastingSpellName, self.playerCastingSpellId, nil, playerName, self.playerCastingTarget, ActionType.START_CAST)
		end
	end
end

function Events:UNIT_SPELLCAST_STOP(unitId, castGuid, spellId)
	if unitId == "player" then
		self.usedStartCast = false
	end
end

-- the player started or ended a channeling spell (like hurricane)
function Events:UNIT_SPELLCAST_CHANNEL_START(unitId, castGuid, spellId)
	if unitId == "player" then
		AttemptReaction((GetSpellInfo(spellId)), spellId, nil, playerName, nil, ActionType.START_CAST)
	end
end

function Events:UNIT_SPELLCAST_CHANNEL_STOP(unitId, castGuid, spellId)
	if unitId == "player" then
		AttemptReaction((GetSpellInfo(spellId)), spellId, nil, playerName, nil, ActionType.CHANNEL_STOP)
	end
end
