local _, RS = ...
RS.Settings = {}
local Settings = RS.Settings

local VERSION = "1.0.6 (TBC Classic)"
local VERSION_NUMBER = 6

function Settings:Load()
	local firstUse

	if ReactionsSave == nil then
		firstUse = true
		ReactionsSave = {}
	end
	settings = ReactionsSave

	if settings.enabled          == nil then settings.enabled          = false end
	if settings.testing          == nil then settings.testing          = false end
	if settings.globalCooldown   == nil then settings.globalCooldown   = 180   end
	if settings.messageCooldown  == nil then settings.messageCooldown  = 3600  end
	if settings.automaticMinimum == nil then settings.automaticMinimum = 30    end
	if settings.automaticMaximum == nil then settings.automaticMaximum = 90    end
	if settings.lowHealthBegin   == nil then settings.lowHealthBegin   = 25    end
	if settings.lowHealthEnd     == nil then settings.lowHealthEnd     = 75    end
	if settings.lowManaBegin     == nil then settings.lowManaBegin     = 20    end
	if settings.lowManaEnd       == nil then settings.lowManaEnd       = 75    end
	if settings.fightLength      == nil then settings.fightLength      = 10    end
	if settings.lowDurability    == nil then settings.lowDurability    = 25    end
	if settings.defaultChance    == nil then settings.defaultChance    = 100   end
	if settings.chanceMultiplier == nil then settings.chanceMultiplier = 1     end
	if settings.groupList        == nil then settings.groupList        = {["Ungrouped"]=true} end
	if settings.tagList          == nil then settings.tagList          = {}    end
	if settings.reactionList     == nil then settings.reactionList     = {}    end
	if settings.chatList         == nil then settings.chatList         = {}    end
	if settings.groupCounter     == nil then settings.groupCounter     = 0     end
	if settings.fightCounter     == nil then settings.fightCounter     = 0     end
	if settings.quietStealth     == nil then settings.quietStealth     = true  end
	if settings.shoutMode        == nil then settings.shoutMode        = false end
	if settings.emoteMode        == nil then settings.emoteMode        = true  end
	if settings.roleItems        == nil then settings.roleItems        = {}    end
	if settings.history          == nil then settings.history          = {}    end

	chatList = ReactionsSave.chatList
	if chatList["trigger"] == nil then chatList["trigger"] = {} end
	if chatList["channel"] == nil then chatList["channel"] = {} end
	if chatList.channel["Action"]       == nil then chatList.channel["Action"]       = {} end
	if chatList.channel["Channel"]      == nil then chatList.channel["Channel"]      = {} end
	if chatList.channel["Emote"]        == nil then chatList.channel["Emote"]        = {} end
	if chatList.channel["Error"]        == nil then chatList.channel["Error"]        = {} end
	if chatList.channel["Guild"]        == nil then chatList.channel["Guild"]        = {} end
	if chatList.channel["Instance"]     == nil then chatList.channel["Instance"]     = {} end
	if chatList.channel["Loot"]         == nil then chatList.channel["Loot"]         = {} end
	if chatList.channel["Officer"]      == nil then chatList.channel["Officer"]      = {} end
	if chatList.channel["Party"]        == nil then chatList.channel["Party"]        = {} end
	if chatList.channel["Raid"]         == nil then chatList.channel["Raid"]         = {} end
	if chatList.channel["Say"]          == nil then chatList.channel["Say"]          = {} end
	if chatList.channel["System"]       == nil then chatList.channel["System"]       = {} end
	if chatList.channel["Tradeskill"]   == nil then chatList.channel["Tradeskill"]   = {} end
	if chatList.channel["Whisper"]      == nil then chatList.channel["Whisper"]      = {} end
	if chatList.channel["Yell"]         == nil then chatList.channel["Yell"]         = {} end
	if chatList.channel["Disabled"]     == nil then chatList.channel["Disabled"]     = {} end

	-- create some default tags
	if firstUse then
		settings.tagList["eightball"]        = {submenu="Random",    amount=20, text="It is certain|It is decidedly so|Without a doubt|Yes definitely|You may rely on it|As I see it, yes|Most likely|Outlook good|Yes|Signs point to yes|Reply hazy try again|Ask again later|Better not tell you now|Cannot predict now|Concentrate and ask again|Don't count on it|My reply is no|My sources say no|Outlook not so good|Very doubtful"}
		settings.tagList["race"]             = {submenu="Character", amount=10, text="draenei|blood elf|dwarf|orc|gnome|tauren|human|troll|night elf|undead"}
		settings.tagList["class"]            = {submenu="Character", amount=9,  text="druid|hunter|mage|paladin|priest|rogue|shaman|warlock|warrior"}
		settings.tagList["profession"]       = {submenu="Character", amount=13, text="alchemy|blacksmithing|enchanting|engineering|herbalism|jewelcrafting|leatherworking|mining|skinning|tailoring|cooking|first aid|fishing"}
		settings.tagList["zone"]             = {submenu="Locations", amount=53, text="Azuremyst Isle|Dun Morogh|Durotar|Elwynn Forest|Eversong Woods|Mulgore|Teldrassil|Tirisfal Glades|Bloodmyst Isle|Darkshore|Ghostlands|Loch Modan|Silverpine Forest|Westfall|Barrens|Redridge Mountains|Stonetalon Mountains|Ashenvale|Duskwood|Hillsbrad Foothills|Wetlands|Thousand Needles|Alterac Mountains|Arathi Highlands|Desolace|Stranglethorn Vale|Dustwallow Marsh|Badlands|Swamp of Sorrows|Feralas|Hinterlands|Tanaris|Searing Gorge|Azshara|Blasted Lands|Un'goro Crater|Felwood|Burning Steppes|Western Plaguelands|Eastern Plaguelands|Winterspring|Moonglade|Silithus|Hellfire Peninsula|Zangarmarsh|Terokkar Forest|Nagrand|Blade's Edge Mountains|Netherstorm|Shadowmoon Valley|Deadwind Pass|Isle of Quel'Danas"}
		settings.tagList["outland"]          = {submenu="Locations", amount=8,  text="Blade's Edge Mountains|Hellfire Peninsula|Nagrand|Netherstorm|Shadowmoon Valley|Skettis|Terokkar Forest|Zangarmarsh"}
		settings.tagList["instance"]         = {submenu="Locations", amount=34, text="Auchenai Crypts|Blackfathom Deeps|Blackrock Depths|Blackrock Spire|Caverns of Time|Dire Maul|Gnomeregan|Hellfire Ramparts|Magisters' Terrace|Mana-Tombs|Maraudon|Old Hillsbrad Foothills|Ragefire Chasm|Razorfen Downs|Razorfen Kraul|Scarlet Monastery|Scholomance|Sethekk Halls|Shadow Labyrinth|Shadowfang Keep|Stratholme|Sunken Temple|the Arcatraz|the Black Morass|the Blood Furnace|the Botanica|the Mechanar|the Shattered Halls|the Slave Pens|the Steamvault|the Underbog|Uldaman|Wailing Caverns|Zul'Farrak"}
		settings.tagList["raid"]             = {submenu="Locations", amount=14, text="Blackwing Lair|Hyjal Summit|Karazhan|Magtheridon's Lair|Molten Core|Naxxramas|Onyxia's Lair|Ruins of Ahn'Qiraj|Sunwell Plateau|Temple of Ahn'Qiraj|The Black Temple|The Eye|Zul'Aman|Zul'Gurub"}
		settings.tagList["battleground"]     = {submenu="Locations", amount=4,  text="Alterac Valley|Arathi Basin|Eye of the Storm|Warsong Gulch"}
		settings.tagList["horde capital"]    = {submenu="Locations", amount=4,  text="Orgrimmar|Thunder Bluff|Silvermoon City|Undercity"}
		settings.tagList["alliance capital"] = {submenu="Locations", amount=4,  text="Stormwind|Ironforge|Darnassus|The Exodar"}
	end

	-- remove old chat trigger person cooldowns
	local time = time
	for name,info in pairs(chatList) do
		if info.personCooldownList then
			for person,usedTime in pairs(info.personCooldownList) do
				if not info.personCooldown or usedTime <= time() - info.personCooldown then
					info.personCooldownList[person] = nil
				end
			end
			if next(info.personCooldownList) == nil then
				info.personCooldownList = nil
			end
		end
	end

	-- update the settings if needed
	self:Upgrade()

	-- set the version at the end after any possible upgrading
	settings.version = VERSION
	settings.versionNumber = VERSION_NUMBER
end

function Settings:Upgrade()
	local settings = ReactionsSave

	if not settings.versionNumber then
		self:Upgrade_1_0_1()
	end
end

-- location pvp conditions are now lowercase
-- set a versionNumber for easier upgrading
function Settings:Upgrade_1_0_1()
	ReactionsSave.versionNumber = 1

	local reaction
	for spellKey,spellValue in pairs(ReactionsSave.reactionList) do -- Spell/Event
		for actionKey,actionValue in pairs(spellValue) do -- actions
			if type(actionValue) == "table" then -- looking for you_hit/you_miss/etc action tables, not other settings
				local actionReactions = actionValue["reactions"]
				if actionReactions then
					for i=1,#actionReactions do -- reaction list for the action
						reaction = actionReactions[i]
						if reaction[5] then
							if reaction[5]["Friendly"] then
								reaction[5]["friendly"] = reaction[5]["Friendly"]
								reaction[5]["Friendly"] = nil
							end
							if reaction[5]["Hostile"] then
								reaction[5]["hostile"] = reaction[5]["Hostile"]
								reaction[5]["Hostile"] = nil
							end
							if reaction[5]["Contested"] then
								reaction[5]["contested"] = reaction[5]["Contested"]
								reaction[5]["Contested"] = nil
							end
							if reaction[5]["Sanctuary"] then
								reaction[5]["sanctuary"] = reaction[5]["Sanctuary"]
								reaction[5]["Sanctuary"] = nil
							end
						end
					end
				end
			end
		end
	end
end
