local _, RS = ...
RS.Variables = {}
local Variables = RS.Variables

-- local references
local UnitName = UnitName
local playerName = UnitName("player")

----------------------------------------------------------------------------------------------------
-- event and message variables
----------------------------------------------------------------------------------------------------
Variables.casterName        = nil
Variables.targetName        = nil
Variables.extraTargetName   = nil
Variables.memberName        = nil
Variables.spellName         = nil
Variables.spellId           = nil
Variables.extraSpellName    = nil      -- for actions with 2 spells, like "Kick" interrupting "Heal"
Variables.extraSpellId      = nil      -- for actions with 2 spells, like "Kick" interrupting "Heal" -- no spell IDs in Classic
Variables.chatMessage       = nil
Variables.chatChannel       = nil
Variables.chatChannelNumber = nil      -- the number of a numbered channel chat type, like 1 for General Chat
Variables.eventDirection    = "nearby" -- a direction description for things like minimap pings
Variables.itemName          = nil      -- name of an item in actions like destroying an item

-- clear names after using them so that messages aren't polluted with bad values
function Variables:ClearNames()
	self.targetName      = nil
	self.casterName      = nil
	self.memberName      = nil
	self.extraTargetName = nil
end

Variables.playerLevel = nil -- set by leveling up event because UnitLevel() can be outdated

--------------------------------------------------
-- simple symbols - parsed separately
--------------------------------------------------
Variables.symbols = {
	["<tm>"]    = "™",
	["<r>"]     = "®",
	["<c>"]     = "©",
	["<cross>"] = "†",
	["<lts>"]   = "<",
	["<gts>"]   = ">",
	["<opar>"]  = "(",
	["<cpar>"]  = ")",
	["<obra>"]  = "{",
	["<cbra>"]  = "}",
	["<pipe>"]  = "||",
}

--------------------------------------------------
-- helper functions
--------------------------------------------------
-- try to find a unitid based on a name
function Variables:FindUnitID(name)
	return    (UnitExists(name) and name)
	       or ((UnitName("target"))    == name and "target")
	       or ((UnitName("mouseover")) == name and "mouseover")
	       or ((UnitName("focus"))     == name and "focus")
	       or (memberName and (UnitName(memberName.."-target")) == name and (memberName.."-target"))
	       or (UnitExists("pet") and (UnitName("pettarget")) == name and "pettarget")
	       or nil
end

-- return the name of a random party member
local function RandomPartyMember(excludePlayer)
	if IsInGroup() then
		local chosen = random(1, GetNumSubgroupMembers() + (excludePlayer and 0 or 1))
		if chosen == GetNumSubgroupMembers() + 1 then
			return UnitName("player")
		end
		return UnitName("party"..chosen)
	end
	return nil
end

-- return the name of a random raid member
local function RandomRaidMember(excludePlayer)
	local playerName = UnitName("player")
	if IsInRaid() then
		local chosen = random(1, GetNumGroupMembers() - (excludePlayer and 1 or 0))
		local name
		for i=1,40 do
			name = (GetRaidRosterInfo(i))
			if name and (not excludePlayer or name ~= playerName) then
				chosen = chosen - 1
				if  chosen == 0 then
					return name
				end
			end
		end
	end
	return nil
end

--------------------------------------------------
-- unitid functions used by multiple variables
--------------------------------------------------
local function VariableRace(unitid)
	if not unitid or unitid == "" then
		return "sturdy being"
	elseif UnitIsPlayer(unitid) then
		return UnitRace(unitid):lower()
	else
		local creatureType = UnitCreatureType(unitid)
		return (creatureType and creatureType ~= "Not specified" and creatureType:lower()) or "mysterious being"
	end
end

local function VariableClass(unitid)
	if not unitid or unitid == "" then
		return "champion for their cause"
	elseif UnitIsPlayer(unitid) then
		return UnitClass(unitid):lower()
	else
		local classification = (UnitClassification(unitid) or "normal"):lower()
		if classification == "worldboss" then
			return "boss"
		elseif classification == "rareelite" then
			return "rare-elite"
		elseif classification == "normal" or classification == "trivial" then
			return classification .. " being"
		else
			return classification
		end
	end
end

local function VariableGuild(unitid)
	local guild = unitid and GetGuildInfo(unitid)
	return guild or "The Organization"
end

local function VariableHeShe(unitid)
	local gender = unitid and UnitSex(unitid)
	return (gender == 2 and "he") or (gender == 3 and "she") or "it"
end
local function VariableHimHer(unitid)
	local gender = unitid and UnitSex(unitid)
	return (gender == 2 and "him") or (gender == 3 and "her") or "it"
end
local function VariableHisHer(unitid)
	local gender = unitid and UnitSex(unitid)
	return (gender == 2 and "his") or (gender == 3 and "her") or "their"
end
local function VariableGender(unitid, pattern)
	local male, female, other = pattern:match("(.-):(.-):(.+)")
	local gender = unitid and UnitSex(unitid)
	return (gender == 2 and (male or "male")) or (gender == 3 and (female or "female")) or other or "thing"
end

--------------------------------------------------
-- list of variables
--------------------------------------------------
local variableFunctions = {
	-- player section
	["player_name"]  = function() return playerName end,
	["player_name_title"] = function()
		local id = GetCurrentTitle() or 0
		if id == 0 then
			return playerName
		elseif id == 36 or id == 38 or id == 39 then -- titles that use suffix instead of prefix
			return playerName .. (id == 38 and " " or ", ") .. GetTitleName(id)
		else
			return GetTitleName(id) .. " " .. playerName
		end
	end,
	["player_race"]       = function() return UnitRace("player"):lower() end,
	["player_class"]      = function() return UnitClass("player"):lower() end,
	["player_guild"]      = function() return GetGuildInfo("player") end,
	["player_title"]      = function() local id = GetCurrentTitle(); return id and GetTitleName(id) or "champion" end,
	["player_hearth"]     = function() return GetBindLocation() or "Parts Unknown" end,
	["player_gold"]       = function() return math.floor(GetMoney() / 10000) end,
	["player_level"]      = function() return Variables.playerLevel or UnitLevel("player") end,
	["player_money_text"] = function() return GetCoinText(GetMoney()) end,
	["pet_name"]          = function() return UnitName("pet") or "my good pet" end,
	-- event targets section
	["target_name"]    = function() return Variables.targetName or "secret target" end,
	["target_race"]    = function() return VariableRace(Variables:FindUnitID(Variables.targetName)) end,
	["target_class"]   = function() return VariableClass(Variables:FindUnitID(Variables.targetName)) end,
	["target_guild"]   = function() return VariableGuild(Variables:FindUnitID(Variables.targetName)) end,
	["target_he_she"]  = function() return VariableHeShe(Variables:FindUnitID(Variables.targetName)) end,
	["target_him_her"] = function() return VariableHimHer(Variables:FindUnitID(Variables.targetName)) end,
	["target_his_her"] = function() return VariableHisHer(Variables:FindUnitID(Variables.targetName)) end,
	["target_gender"]  = function(pattern) return VariableGender(Variables:FindUnitID(Variables.targetName), pattern) end,

	["extra_target_name"]    = function() return Variables.extraTargetName or "someone who probably deserves it" end,
	["extra_target_race"]    = function() return VariableRace(Variables:FindUnitID(Variables.extraTargetName)) end,
	["extra_target_class"]   = function() return VariableClass(Variables:FindUnitID(Variables.extraTargetName)) end,
	["extra_target_guild"]   = function() return VariableGuild(Variables:FindUnitID(Variables.extraTargetName)) end,
	["extra_target_he_she"]  = function() return VariableHeShe(Variables:FindUnitID(Variables.extraTargetName)) end,
	["extra_target_him_her"] = function() return VariableHimHer(Variables:FindUnitID(Variables.extraTargetName)) end,
	["extra_target_his_her"] = function() return VariableHisHer(Variables:FindUnitID(Variables.extraTargetName)) end,
	["extra_target_gender"]  = function(pattern) return VariableGender(Variables:FindUnitID(Variables.extraTargetName), pattern) end,

	["group_name"]    = function() return Variables.memberName or "friendly teammate" end,
	["group_race"]    = function() return VariableRace(Variables:FindUnitID(Variables.memberName)) end,
	["group_class"]   = function() return VariableClass(Variables:FindUnitID(Variables.memberName)) end,
	["group_guild"]   = function() return VariableGuild(Variables:FindUnitID(Variables.memberName)) end,
	["group_he_she"]  = function() return VariableHeShe(Variables:FindUnitID(Variables.memberName)) end,
	["group_him_her"] = function() return VariableHimHer(Variables:FindUnitID(Variables.memberName)) end,
	["group_his_her"] = function() return VariableHisHer(Variables:FindUnitID(Variables.memberName)) end,
	["group_gender"]  = function(pattern) return VariableGender(Variables:FindUnitID(Variables.memberName), pattern) end,

	-- event info section
	["spell_link"]        = function() return (Variables.spellId and GetSpellLink(Variables.spellId)) or (Variables.spellName and GetSpellLink(Variables.spellName)) or GetSpellLink(39477) end, -- 39477 = "Bad Luck"
	["spell_name"]        = function() return Variables.spellName or "Bad Luck Aura" end,
	["spell_rank"]        = function(spell) -- name or ID
		spell = spell ~= "" and spell or Variables.spellId
		local rank = spell and (select(2, GetSpellInfo(spell)))
		return rank and rank ~= "" and rank:lower() or "unrankable rank"
	end,
	["extra_spell_link"]  = function() return (Variables.extraSpellId and GetSpellLink(Variables.extraSpellId)) or (Variables.extraSpellName and GetSpellLink(Variables.extraSpellName)) or GetSpellLink(39477) end, -- 39477 = "Bad Luck"
	["extra_spell_name"]  = function() return Variables.extraSpellName or "Bad Luck Aura" end,
	["extra_spell_rank"]  = function() return tonumber(Variables.extraSpellId) and (select(2, GetSpellInfo(Variables.extraSpellId))):lower() or "unrankable rank" end,
	["spell_name_after"]  = function(prefix) return (Variables.spellName and Variables.spellName:match(prefix.."(.+)") or Variables.spellName) or "Bad Luck Aura" end,

	-- macro-style targets
	["name"]    = function(unitid)  return UnitName(unitid or "none") or "mystery person" end,
	["race"]    = function(unitid)  return VariableRace(unitid) end,
	["class"]   = function(unitid)  return VariableClass(unitid) end,
	["guild"]   = function(Unitid)  return VariableGuild(unitid) end,
	["he_she"]  = function(unitid)  return VariableHeShe(unitid) end,
	["him_her"] = function(unitid)  return VariableHimHer(unitid) end,
	["his_her"] = function(unitid)  return VariableHisHer(unitid) end,
	["gender"]  = function(unitid, pattern) return VariableGender(unitid, pattern) end,

	-- equipment
	["eq_head_link"]     = function() return GetInventoryItemLink("player",  1) or "invisible helmet"          end,
	["eq_neck_link"]     = function() return GetInventoryItemLink("player",  2) or "invisible necklace"        end,
	["eq_shoulder_link"] = function() return GetInventoryItemLink("player",  3) or "invisible shoulderpads"    end,
	["eq_back_link"]     = function() return GetInventoryItemLink("player", 15) or "invisible cape"            end,
	["eq_chest_link"]    = function() return GetInventoryItemLink("player",  5) or "invisible chestpiece"      end,
	["eq_shirt_link"]    = function() return GetInventoryItemLink("player",  4) or "invisible shirt"           end,
	["eq_tabard_link"]   = function() return GetInventoryItemLink("player", 19) or "invisible tabard"          end,
	["eq_wrist_link"]    = function() return GetInventoryItemLink("player",  9) or "invisible wristbands"      end,
	["eq_hands_link"]    = function() return GetInventoryItemLink("player", 10) or "invisible gloves"          end,
	["eq_waist_link"]    = function() return GetInventoryItemLink("player",  6) or "invisible belt"            end,
	["eq_legs_link"]     = function() return GetInventoryItemLink("player",  7) or "invisible pants"           end,
	["eq_feet_link"]     = function() return GetInventoryItemLink("player",  8) or "invisible boots"           end,
	["eq_finger1_link"]  = function() return GetInventoryItemLink("player", 11) or "invisible ring"            end,
	["eq_finger2_link"]  = function() return GetInventoryItemLink("player", 12) or "invisible ring"            end,
	["eq_trinket1_link"] = function() return GetInventoryItemLink("player", 13) or "invisible trinket"         end,
	["eq_trinket2_link"] = function() return GetInventoryItemLink("player", 14) or "invisible trinket"         end,
	["eq_mainhand_link"] = function() return GetInventoryItemLink("player", 16) or "invisible weapon"          end,
	["eq_offhand_link"]  = function() return GetInventoryItemLink("player", 17) or "invisible handheld object" end,
	["eq_ranged_link"]   = function() return GetInventoryItemLink("player", 18) or "invisible weapon or relic" end,
	["eq_ammo_link"]     = function() return GetInventoryItemLink("player",  0) or "invisible ammo"            end,
	["eq_head_name"]     = function() local link = GetInventoryItemLink("player",  1) return link and link:match("%[(.-)]") or "invisible helmet"          end,
	["eq_neck_name"]     = function() local link = GetInventoryItemLink("player",  2) return link and link:match("%[(.-)]") or "invisible necklace"        end,
	["eq_shoulder_name"] = function() local link = GetInventoryItemLink("player",  3) return link and link:match("%[(.-)]") or "invisible shoulderpads"    end,
	["eq_back_name"]     = function() local link = GetInventoryItemLink("player", 15) return link and link:match("%[(.-)]") or "invisible cape"            end,
	["eq_chest_name"]    = function() local link = GetInventoryItemLink("player",  5) return link and link:match("%[(.-)]") or "invisible chestpiece"      end,
	["eq_shirt_name"]    = function() local link = GetInventoryItemLink("player",  4) return link and link:match("%[(.-)]") or "invisible shirt"           end,
	["eq_tabard_name"]   = function() local link = GetInventoryItemLink("player", 19) return link and link:match("%[(.-)]") or "invisible tabard"          end,
	["eq_wrist_name"]    = function() local link = GetInventoryItemLink("player",  9) return link and link:match("%[(.-)]") or "invisible wristbands"      end,
	["eq_hands_name"]    = function() local link = GetInventoryItemLink("player", 10) return link and link:match("%[(.-)]") or "invisible gloves"          end,
	["eq_waist_name"]    = function() local link = GetInventoryItemLink("player",  6) return link and link:match("%[(.-)]") or "invisible belt"            end,
	["eq_legs_name"]     = function() local link = GetInventoryItemLink("player",  7) return link and link:match("%[(.-)]") or "invisible pants"           end,
	["eq_feet_name"]     = function() local link = GetInventoryItemLink("player",  8) return link and link:match("%[(.-)]") or "invisible boots"           end,
	["eq_finger1_name"]  = function() local link = GetInventoryItemLink("player", 11) return link and link:match("%[(.-)]") or "invisible ring"            end,
	["eq_finger2_name"]  = function() local link = GetInventoryItemLink("player", 12) return link and link:match("%[(.-)]") or "invisible ring"            end,
	["eq_trinket1_name"] = function() local link = GetInventoryItemLink("player", 13) return link and link:match("%[(.-)]") or "invisible trinket"         end,
	["eq_trinket2_name"] = function() local link = GetInventoryItemLink("player", 14) return link and link:match("%[(.-)]") or "invisible trinket"         end,
	["eq_mainhand_name"] = function() local link = GetInventoryItemLink("player", 16) return link and link:match("%[(.-)]") or "invisible weapon"          end,
	["eq_offhand_name"]  = function() local link = GetInventoryItemLink("player", 17) return link and link:match("%[(.-)]") or "invisible handheld object" end,
	["eq_ranged_name"]   = function() local link = GetInventoryItemLink("player", 18) return link and link:match("%[(.-)]") or "invisible weapon or relic" end,
	["eq_ammo_name"]     = function() local link = GetInventoryItemLink("player",  0) return link and link:match("%[(.-)]") or "invisible ammo"            end,

	-- random section
	["number"] = function(text)
		local low, high = text:match("%s*([%-%d]+)%s*([%-%d]*)")
		low, high = tonumber(low), tonumber(high)
		if not low then
			return random(1, 100)
		elseif not high then
			low, high = 1, low
		end
		if low > high then
			low, high = high, low
		end
		return random(low, high)
	end,
	["random_target_icon"]    = function() return "{rt" .. math.random(1, 8) .. "}" end,
	["random_guild_member"]   = function() return RandomGuildMember(true) or "my imaginary friend" end,
	["random_party_member"]   = function() return RandomPartyMember(true) or "my imaginary friend" end,
	["random_group_member"]   = function()
		local name
		if IsInRaid() > 0 then
			name = RandomRaidMember(true)
		else
			name = RandomPartyMember(true)
		end
		return name or "my imaginary friend"
	end,
	["random_roll"] = function() return random(1, 100) end,
	["random_tutorial_message"] = function() return _G["TUTORIAL"..random(1,51)]:gsub("|c%x%x%x%x%x%x%x%x", ""):gsub("|r",""):gsub("\n", " ") end,
	-- miscellaneous
	["make_spell_link"] = function(id) return tonumber(id) and GetSpellLink(id) or GetSpellLink(39477) end, -- 39477 = "Bad Luck"
	["zone"]      = function() local zone = GetRealZoneText() return zone ~= "" and zone or "The Land Down Under" end,
	["subzone"]   = function() local subzone = GetSubZoneText() return subzone ~= "" and subzone or "Parts Unknown" end,
	["zone_full"] = function()
		local zone = GetRealZoneText()
		local subzone = GetSubZoneText()
		if not zone or zone == "" then
			return "The Land Down Under"
		elseif not subzone or subzone == "" or subzone == zone then
			return zone
		else
			return subzone .. " in " .. zone
		end
	end,
	["coords"]       = function() local x,y = GetPlayerMapPosition("player"); if x==0.0 and y==0.0 then return "a secret location" end return string.format("%.0f, %.0f", x*100, y*100) end,
	["coords_exact"] = function() local x,y = GetPlayerMapPosition("player"); if x==0.0 and y==0.0 then return "a secret location" end return string.format("%.1f, %.1f", x*100, y*100) end,
	["summon_last_zone"] = function() return C_SummonInfo.GetSummonConfirmAreaName() or "Twisting Nether" end,
	["summon_time_left"] = function() return C_SummonInfo.GetSummonConfirmTimeLeft() or 0 end,
	["item_name"] = function() return Variables.itemName or "a tricky item" end,
	["item_link"] = function() return Variables.itemLink or "a tricky item" end,
	["direction"] = function(prefix)
		local direction = Variables.eventDirection
		if prefix and prefix ~= "" and direction ~= "nearby" then
			return prefix .. " " .. direction
		end
		return direction
	end,
	["game_time_simple"] = function()
		local hour = GetGameTime()
		if     hour < 5  then return "early morning"
		elseif hour < 12 then return "morning"
		elseif hour < 17 then return "afternoon"
		else                  return "evening"
		end
	end,
	["real_time_simple"] = function()
		local hour = date("*t").hour
		if     hour < 5  then return "early morning"
		elseif hour < 12 then return "morning"
		elseif hour < 17 then return "afternoon"
		else                  return "evening"
		end
	end,
	["game_time_general"] = function()
		local hour = GetGameTime()
		if     hour < 5 or hour >= 21 then return "night"
		elseif hour < 12              then return "morning"
		elseif hour < 17              then return "afternoon"
		else                               return "evening"
		end
	end,
	["real_time_general"] = function()
		local hour = date("*t").hour
		if     hour < 5 or hour >= 21 then return "night"
		elseif hour < 12              then return "morning"
		elseif hour < 17              then return "afternoon"
		else                               return "evening"
		end
	end,
	["game_time_description"] = function()
		local hour, minute = GetGameTime()
		if     (hour == 0 and minute < 15) or (hour == 23 and minute >= 45) then return "around midnight"
		elseif hour < 5                  then return "late at night"
		elseif hour < 7                  then return "early in the morning"
		elseif hour < 12 and minute < 45 then return "in the morning"
		elseif hour < 13 and minute < 15 then return "around noon"
		elseif hour < 16                 then return "in the afternoon"
		elseif hour < 17                 then return "in the late afternoon"
		elseif hour < 20                 then return "in the evening"
		elseif hour < 21                 then return "late in the evening"
		else                                  return "at night"
		end
	end,
	["real_time_description"] = function()
		local currentTime = date("*t")
		local hour = currentTime.hour
		local minute = currentTime.min
		if     (hour == 0 and minute < 15) or (hour == 23 and minute >= 45) then return "around midnight"
		elseif hour < 5                  then return "late at night"
		elseif hour < 7                  then return "early in the morning"
		elseif hour < 12 and minute < 45 then return "in the morning"
		elseif hour < 13 and minute < 15 then return "around noon"
		elseif hour < 16                 then return "in the afternoon"
		elseif hour < 17                 then return "in the late afternoon"
		elseif hour < 20                 then return "in the evening"
		elseif hour < 21                 then return "late in the evening"
		else                                  return "at night"
		end
	end,
	-- chat
	["channel"] = function() return Variables.chatChannel or "tunnel of puzzles" end,
	["message"] = function() return Variables.chatMessage or "an alphabet in another language" end,
	["channel_number"] = function() return Variables.chatChannelNumber or "0" end,
	["capture"] = function(index) return rs.capture[tonumber(index) or "none"] or "Unknown" end,
}

-- aliases for <extra_*> names
variableFunctions["target_name2"] = variableFunctions["extra_target_name"]
variableFunctions["target_race2"] = variableFunctions["extra_target_race"]
variableFunctions["target_class2"] = variableFunctions["extra_target_class"]
variableFunctions["target_guild2"] = variableFunctions["extra_target_guild"]
variableFunctions["target_he_she2"] = variableFunctions["extra_target_he_she"]
variableFunctions["target_him_her2"] = variableFunctions["extra_target_him_her"]
variableFunctions["target_his_her2"] = variableFunctions["extra_target_his_her"]
variableFunctions["target_gender2"] = variableFunctions["extra_target_gender"]
variableFunctions["spell_link2"] = variableFunctions["extra_spell_link"]
variableFunctions["spell_name2"] = variableFunctions["extra_spell_name"]
variableFunctions["spell_rank2"] = variableFunctions["extra_spell_rank"]

if DataHorde then
	local DataHorde = DataHorde
	variableFunctions["random_item"]   = function() return DataHorde.GetRandomItem() end
	variableFunctions["random_quest"]  = function() return DataHorde.GetRandomQuest() end
	variableFunctions["random_npc"]    = function() return DataHorde.GetRandomNPC() end
	variableFunctions["random_spell"]  = function() return DataHorde.GetRandomSpell() end
	variableFunctions["random_pspell"] = function(class) return DataHorde.GetRandomPlayerSpell(class) end
	variableFunctions["random_craft"]  = function(profession) return DataHorde.GetRandomCraft(profession) or "handiwork craftsmanship" end

	variableFunctions["random_talent"]     = function(class) return DataHorde.GetRandomTalent(class)       or "magical maneuver" end
	variableFunctions["random_talent_min"] = function(class) return DataHorde.GetRandomTalent(class, 0, 0) or "magical maneuver" end
	variableFunctions["random_talent_max"] = function(class) return DataHorde.GetRandomTalent(class, 5, 5) or "magical maneuver" end

	variableFunctions["random_faction"]          = function() return DataHorde.GetRandomFaction(true, true, true)   or "no one" end
	variableFunctions["random_alliance_faction"] = function() return DataHorde.GetRandomFaction(false, true, false) or "no one" end
	variableFunctions["random_horde_faction"]    = function() return DataHorde.GetRandomFaction(false, false, true) or "no one" end
	variableFunctions["random_neutral_faction"]  = function() return DataHorde.GetRandomFaction(true, false, false) or "no one" end

	variableFunctions["random_subzone"]                = function() return DataHorde.GetRandomSubzone(true, true, true)        or "Parts Unknown" end
	variableFunctions["random_zone"]                   = function() return DataHorde.GetRandomZone(true,  true,  true,  false) or "Parts Unknown" end
	variableFunctions["random_full_zone"]              = function() return DataHorde.GetRandomZone(true,  true,  true,  true)  or "Parts Unknown" end
	variableFunctions["random_instance_zone"]          = function() return DataHorde.GetRandomZone(false, false, true,  false) or "Parts Unknown" end
	variableFunctions["random_instance_full_zone"]     = function() return DataHorde.GetRandomZone(false, false, true,  true)  or "Parts Unknown" end
	variableFunctions["random_battleground_zone"]      = function() return DataHorde.GetRandomZone(false, true,  false, false) or "Parts Unknown" end
	variableFunctions["random_battleground_full_zone"] = function() return DataHorde.GetRandomZone(false, true,  false, true)  or "Parts Unknown" end
	variableFunctions["random_world_zone"]             = function() return DataHorde.GetRandomZone(true,  false, false, false) or "Parts Unknown" end
	variableFunctions["random_world_full_zone"]        = function() return DataHorde.GetRandomZone(true,  false, false, true)  or "Parts Unknown" end
end

function Variables:Lookup(name, args)
	local func = variableFunctions[name]
	if func then
		return func(args)
	end
end
